#include <Arduino.h>
#include <mrm-8x8a.h>
#include <mrm-board.h>
#include <mrm-bldc2x50.h>
#include <mrm-bldc4x2.5.h>
#include <mrm-can-bus.h>
#include <mrm-col-b.h>
#include <mrm-col-can.h>
#include <mrm-common.h>
#include <mrm-fet-can.h>
#include <mrm-imu.h>
// #include <mrm-ir-finder2.h>
#include <mrm-ir-finder3.h>
// #include <mrm-ir-finder-can.h>
#include <mrm-lid-can-b.h>
#include <mrm-lid-can-b2.h>
#include <mrm-mot2x50.h>
#include <mrm-mot4x10.h>
#include <mrm-mot4x3.6can.h>
#include <mrm-node.h>
#include <mrm-pid.h>
#include <mrm-ref-can.h>
#include <mrm-robot.h>
#include "mrm-robot-line.h"
#include "mrm-robot-maze.h"
#include "mrm-robot-min.h"
#include "mrm-robot-soccer.h"
#include <mrm-servo.h>
#include <mrm-switch.h>
#include <mrm-therm-b-can.h>
// #include <mrm-us.h>
#include <mrm-us-b.h>
#include <mrm-us1.h>

Robot *robot;

#define MIN_MOTOR_SPEED(speed_modifier) (((speed_modifier) >= 0 ? 1 : -1) * MIN_SPEED_THRESHOLD)
#define MAX_MOTOR_SPEED(speed_modifier) (((speed_modifier) >= 0 ? 1 : -1) * MAX_SPEED_THRESHOLD)

#define SPEED_STEP 10
#define SPEED_BEND_BOOST 1.2
#define TILT_MODIFIER 2
#define MIN_SPEED_THRESHOLD 15
#define MAX_SPEED_THRESHOLD 105
#define TIGHT_BEND_BOOST 10
#define DIRECTION_FORWARDS 1
#define DIRECTION_BACKWARDS -1

bool init_private = false;
int speed_modifier = 0;

int leftmotor = speed_modifier;
int rightmotor = speed_modifier;
int last_direction = DIRECTION_FORWARDS;

char msg_private[2];
char prev[2];

void setup()
{
	// U konstruktoru OBAVEZNO dodijelite robotu ime kako piše na njegovoj naljepnici!!
	// Na taj ćemo način smo sigurni da se nikoja dva robota ne zopvu istim imenom
	// i moći ćemo s njima uspostaviti Bluetooth komunikaciju
	robot = new RobotLine((char *)"My robot"); // RobotLine, RobotMaze, RobotMin, RobotSoccer, or Your custom robot. "My robot" is Bluetooth name.
	robot->print("Start.\n\rOvo je naš prvi program za robota\n\r");
	strcpy(msg_private, "x");
}

void loop()
{
	robot->refresh();
}

/*
Funkcija članica loop pojedinih vrsta robota se izvršava kod svakog prolaska kroz petlju
Pokreće se opisivanjem loo u serijskom monitoru na računalu ili mobitelu
U tu svrhu je korisno imati računalo s Bluetooth vezom.
*/

/*
Definicija funkcije loop se izvorno nalazila u datoteci mrm-robot-line.cpp,
ali sam je premjestio ovdje kako ne bismo na radionici
morali mijenjati datoteku mrm-robot-line.cppp već samo main.cpp
(na radionici ćemo koristiti RobotLine)
*/

// start (loo)
void RobotLine::loop()
{
	strcpy(prev, msg_private);
	strcpy(msg_private, "S");
	display(msg_private);

	if (strcmp(prev, "S") == 0)
	{
		go(leftmotor, rightmotor);
	}
	else
	{
		speed_modifier = MIN_SPEED_THRESHOLD;
		leftmotor = rightmotor = speed_modifier;
	}
}

// tilt - balanced (lo0)
void RobotLine::loop0()
{
	strcpy(prev, msg_private);
	strcpy(msg_private, "A");
	display(msg_private);

	if (strcmp(prev, "A") == 0)
	{
		go(leftmotor, rightmotor);
	}
	else
	{
		leftmotor = rightmotor = speed_modifier;
	}
}

// tilt - left (lo1)
void RobotLine::loop1()
{
	strcpy(prev, msg_private);
	strcpy(msg_private, "C");
	display(msg_private);

	if (strcmp(prev, "C") == 0)
	{
		go(leftmotor, rightmotor);
	}
	else
	{
		leftmotor = speed_modifier / TILT_MODIFIER;
		rightmotor = speed_modifier * TILT_MODIFIER * SPEED_BEND_BOOST;

		if (speed_modifier == 0) // manevar
		{
			leftmotor = 0;
			rightmotor = last_direction * (MIN_SPEED_THRESHOLD + TIGHT_BEND_BOOST);
		}
		else
		{
			if (-MIN_SPEED_THRESHOLD <= leftmotor && leftmotor <= MIN_SPEED_THRESHOLD)
			{
				leftmotor = MIN_MOTOR_SPEED(speed_modifier);
			}

			if (rightmotor < -MAX_SPEED_THRESHOLD || MAX_SPEED_THRESHOLD < rightmotor)
			{
				rightmotor = MAX_MOTOR_SPEED(speed_modifier);
			}
		}
	}
}

// forward (lo2)
void RobotLine::loop2()
{
	strcpy(prev, msg_private);
	strcpy(msg_private, "D");
	display(msg_private);

	if (strcmp(prev, "D") == 0)
	{
		go(leftmotor, rightmotor);
	}
	else
	{
		speed_modifier = MIN_SPEED_THRESHOLD;
		leftmotor = rightmotor = speed_modifier;
	}
}

// tilt - right (lo3)
void RobotLine::loop3()
{
	strcpy(prev, msg_private);
	strcpy(msg_private, "E");
	display(msg_private);

	if (strcmp(prev, "E") == 0)
	{
		go(leftmotor, rightmotor);
	}
	else
	{
		rightmotor = speed_modifier / TILT_MODIFIER;
		leftmotor = speed_modifier * TILT_MODIFIER * SPEED_BEND_BOOST;

		if (speed_modifier == 0) // manevar
		{
			rightmotor = 0;
			leftmotor = last_direction * (MIN_SPEED_THRESHOLD + TIGHT_BEND_BOOST);
		}
		else
		{
			if (-MIN_SPEED_THRESHOLD <= rightmotor && rightmotor <= MIN_SPEED_THRESHOLD)
			{
				rightmotor = MIN_MOTOR_SPEED(speed_modifier);
			}

			if (leftmotor < -MAX_SPEED_THRESHOLD || MAX_SPEED_THRESHOLD < leftmotor)
			{
				leftmotor = MAX_MOTOR_SPEED(speed_modifier);
			}
		}
	}
}

// slower (lo4)
void RobotLine::loop4()
{
	strcpy(prev, msg_private);
	strcpy(msg_private, "F");
	display(msg_private);

	if (strcmp(prev, "F") == 0)
	{
		go(leftmotor, rightmotor);
	}
	else
	{
		float left;
		float right;

		left = (float)leftmotor / speed_modifier;
		right = (float)rightmotor / speed_modifier;
		speed_modifier -= (speed_modifier >= 0 ? 1 : -1) * SPEED_STEP;

		if (-MIN_SPEED_THRESHOLD < speed_modifier && speed_modifier < MIN_SPEED_THRESHOLD) // 15 je min ?
		{
			last_direction = (speed_modifier >= 0) ? DIRECTION_FORWARDS : DIRECTION_BACKWARDS;
			leftmotor = rightmotor = speed_modifier = 0;
		}
		else
		{
			left *= speed_modifier;
			right *= speed_modifier;

			if (-MIN_SPEED_THRESHOLD < left && left < MIN_SPEED_THRESHOLD)
			{
				leftmotor = MIN_MOTOR_SPEED(speed_modifier);
			}
			else if (left < -MAX_SPEED_THRESHOLD || MAX_SPEED_THRESHOLD < left)
			{
				leftmotor = MAX_MOTOR_SPEED(speed_modifier);
			}
			else
			{
				leftmotor = left;
			}

			if (-MIN_SPEED_THRESHOLD < right && right < MIN_SPEED_THRESHOLD)
			{
				rightmotor = MIN_MOTOR_SPEED(speed_modifier);
			}
			else if (right < -MAX_SPEED_THRESHOLD || MAX_SPEED_THRESHOLD < right)
			{
				rightmotor = MAX_MOTOR_SPEED(speed_modifier);
			}
			else
			{
				rightmotor = right;
			}
		}
	}
}

// stop (lo5) -- postoji i BT komanda 'x', ovo je "soft stop" jer zelimo da se moze okretati oko sebe kad stane
void RobotLine::loop5()
{
	strcpy(prev, msg_private);
	strcpy(msg_private, "Q");
	display(msg_private);

	if (strcmp(prev, "Q") != 0) // DRUGACIJE, PAZI
	{
		last_direction = (speed_modifier >= 0) ? DIRECTION_FORWARDS : DIRECTION_BACKWARDS;
		speed_modifier = 0;
		leftmotor = rightmotor = speed_modifier;
		go(leftmotor, rightmotor);
	}
}

// faster (lo6)
void RobotLine::loop6()
{
	strcpy(prev, msg_private);
	strcpy(msg_private, "G");
	display(msg_private);

	if (strcmp(prev, "G") == 0)
	{
		go(leftmotor, rightmotor);
	}
	else
	{
		if (speed_modifier == 0)
		{
			speed_modifier = last_direction * MIN_SPEED_THRESHOLD;
			leftmotor = rightmotor = speed_modifier;
			return;
		}

		float left;
		float right;

		left = (float)leftmotor / speed_modifier;
		right = (float)rightmotor / speed_modifier;
		speed_modifier += (speed_modifier >= 0 ? 1 : -1) * SPEED_STEP;

		if (-MIN_SPEED_THRESHOLD < speed_modifier && speed_modifier < MIN_SPEED_THRESHOLD) // 15 je min ?
		{
			last_direction = (speed_modifier >= 0) ? DIRECTION_FORWARDS : DIRECTION_BACKWARDS;
			leftmotor = rightmotor = speed_modifier = 0;
		}
		else
		{
			left *= speed_modifier;
			right *= speed_modifier;

			if (-MIN_SPEED_THRESHOLD < left && left < MIN_SPEED_THRESHOLD)
			{
				leftmotor = MIN_MOTOR_SPEED(speed_modifier);
			}
			else if (left < -MAX_SPEED_THRESHOLD || MAX_SPEED_THRESHOLD < left)
			{
				leftmotor = MAX_MOTOR_SPEED(speed_modifier);
			}
			else
			{
				leftmotor = left;
			}

			if (-MIN_SPEED_THRESHOLD < right && right < MIN_SPEED_THRESHOLD)
			{
				rightmotor = MIN_MOTOR_SPEED(speed_modifier);
			}
			else if (right < -MAX_SPEED_THRESHOLD || MAX_SPEED_THRESHOLD < right)
			{
				rightmotor = MAX_MOTOR_SPEED(speed_modifier);
			}
			else
			{
				rightmotor = right;
			}
		}
	}
}

// unused
void RobotLine::loop7()
{
}

// reverse (lo8)
void RobotLine::loop8()
{
	strcpy(prev, msg_private);
	strcpy(msg_private, "H");
	display(msg_private);

	if (strcmp(prev, "H") == 0)
	{
		go(leftmotor, rightmotor);
	}
	else
	{
		speed_modifier = -MIN_SPEED_THRESHOLD;
		leftmotor = rightmotor = speed_modifier;
	}
}

// unused
void RobotLine::loop9()
{
}